# coding: utf-8

"""
    Kestra EE

    All API operations, except for Superadmin-only endpoints, require a tenant identifier in the HTTP path.<br/> Endpoints designated as Superadmin-only are not tenant-scoped.

    The version of the OpenAPI document: 1.1.4
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from kestrapy.api.groups_api import GroupsApi


class TestGroupsApi(unittest.TestCase):
    """GroupsApi unit test stubs"""

    def setUp(self) -> None:
        self.api = GroupsApi()

    def tearDown(self) -> None:
        pass

    def test_add_user_to_group(self) -> None:
        """Test case for add_user_to_group

        Add a user to a group
        """
        pass

    def test_autocomplete_groups(self) -> None:
        """Test case for autocomplete_groups

        List groups for autocomplete
        """
        pass

    def test_create_group(self) -> None:
        """Test case for create_group

        Create a group
        """
        pass

    def test_delete_group(self) -> None:
        """Test case for delete_group

        Delete a group
        """
        pass

    def test_delete_user_from_group(self) -> None:
        """Test case for delete_user_from_group

        Remove a user from a group
        """
        pass

    def test_group(self) -> None:
        """Test case for group

        Retrieve a group
        """
        pass

    def test_list_group_ids(self) -> None:
        """Test case for list_group_ids

        List groups by ids
        """
        pass

    def test_search_group_members(self) -> None:
        """Test case for search_group_members

        Search for users in a group
        """
        pass

    def test_search_groups(self) -> None:
        """Test case for search_groups

        Search for groups
        """
        pass

    def test_set_user_membership_for_group(self) -> None:
        """Test case for set_user_membership_for_group

        Update a user's membership type in a group
        """
        pass

    def test_update_group(self) -> None:
        """Test case for update_group

        Update a group
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Kestra EE

    All API operations, except for Superadmin-only endpoints, require a tenant identifier in the HTTP path.<br/> Endpoints designated as Superadmin-only are not tenant-scoped.

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest
from kestrapy import (Configuration,
                      KestraClient,
                      IAMUserControllerApiCreateOrUpdateUserRequest,
                      IAMTenantAccessControllerUserApiAutocomplete,
                      CreateApiTokenRequest,
                      MeControllerApiUpdatePasswordRequest,
                      IAMUserControllerApiPatchUserPasswordRequest,
                      ApiPatchSuperAdminRequest,
                      IAMUserGroupControllerApiUpdateUserGroupsRequest,
                      IAMGroupControllerApiCreateGroupRequest
                      )

class TestUsersApi(unittest.TestCase):
    """UsersApi unit test stubs"""

    def setUp(self) -> None:
        configuration = Configuration()
        configuration.host = "http://localhost:9902"
        configuration.username = "root@root.com"
        configuration.password = "Root!1234"

        self.kestra_client = KestraClient(configuration)
        self.tenant = "main"


    def tearDown(self) -> None:
        pass

    def test_autocomplete_users(self) -> None:
        """Test case for autocomplete_users

        List users for autocomplete
        """
        email = "test_autocomplete_users@kestra.io"
        # create a user to autocomplete
        user = IAMUserControllerApiCreateOrUpdateUserRequest(
            email=email
        )

        created_user = self.kestra_client.users.create_user(
            iam_user_controller_api_create_or_update_user_request=user
        )
        # Create a tenantAccess by adding user to a group
        group = IAMGroupControllerApiCreateGroupRequest(
            name="test_autocomplete_users",
            description="An example group"
        )

        created_group = self.kestra_client.groups.create_group(
            tenant=self.tenant,
            iam_group_controller_api_create_group_request=group
        )

        self.kestra_client.groups.add_user_to_group(
            tenant=self.tenant,
            id=created_group.id,
            user_id=created_user.id
        )

        autocomplete_req = IAMTenantAccessControllerUserApiAutocomplete(q=email)

        results = self.kestra_client.users.autocomplete_users(
            tenant=self.tenant,
            iam_tenant_access_controller_user_api_autocomplete=autocomplete_req
        )

        assert any(getattr(r, 'username', None) == created_user.email for r in results)

    def test_create_api_tokens_for_user(self) -> None:
        """Test case for create_api_tokens_for_user

        Create new API Token for a specific user
        """
        name_base = "test_create_api_tokens_for_user"
        # create user
        user_req = IAMUserControllerApiCreateOrUpdateUserRequest(
            email=f"{name_base}@kestra.io"
        )
        created_user = self.kestra_client.users.create_user(
            iam_user_controller_api_create_or_update_user_request=user_req
        )

        # token name must match regex: use lowercase and hyphens
        token_req = CreateApiTokenRequest(
            name=name_base.replace("_", "-"),
            description="token for test_create_api_tokens_for_user",
        )
        token = self.kestra_client.users.create_api_tokens_for_user(
            id=created_user.id,
            create_api_token_request=token_req
        )

        assert getattr(token, 'id', None) is not None

    def test_create_user(self) -> None:
        """Test case for create_user

        Create a new user account
        """
        name_base = "test_create_user"
        user_req = IAMUserControllerApiCreateOrUpdateUserRequest(
            email=f"{name_base}@kestra.io",
            first_name=name_base,
            password="Password!234"
        )

        created = self.kestra_client.users.create_user(
            iam_user_controller_api_create_or_update_user_request=user_req
        )
        assert created.email == user_req.email

        # cleanup
        try:
            self.kestra_client.users.delete_user(id=created.id)
        except Exception:
            pass

    def test_delete_api_token_for_user(self) -> None:
        """Test case for delete_api_token_for_user

        Delete an API Token for specific user and token id
        """
        name_base = "test_delete_api_token_for_user"
        user_req = IAMUserControllerApiCreateOrUpdateUserRequest(
            email=f"{name_base}@kestra.io"
        )
        created_user = self.kestra_client.users.create_user(
            iam_user_controller_api_create_or_update_user_request=user_req
        )

        token_req = CreateApiTokenRequest(
            name=name_base.replace("_", "-"),
            description="token to delete",
        )
        token = self.kestra_client.users.create_api_tokens_for_user(id=created_user.id, create_api_token_request=token_req)

        self.kestra_client.users.delete_api_token_for_user(id=created_user.id, token_id=token.id)

    def test_delete_refresh_token(self) -> None:
        """Delete a user refresh token

        This operation is superadmin-only; create a user and attempt to delete refresh token (may be no-op)
        """
        name_base = "test_delete_refresh_token"
        user_req = IAMUserControllerApiCreateOrUpdateUserRequest(
            email=f"{name_base}@kestra.io"
        )
        created_user = self.kestra_client.users.create_user(
            iam_user_controller_api_create_or_update_user_request=user_req
        )

        # call delete_refresh_token (no explicit token id required)
        self.kestra_client.users.delete_refresh_token(id=created_user.id)

        # cleanup
        try:
            self.kestra_client.users.delete_user(id=created_user.id)
        except Exception:
            pass

    def test_delete_user(self) -> None:
        """Delete a user"""
        name_base = "test_delete_user"
        user_req = IAMUserControllerApiCreateOrUpdateUserRequest(
            email=f"{name_base}@kestra.io"
        )
        created_user = self.kestra_client.users.create_user(
            iam_user_controller_api_create_or_update_user_request=user_req
        )

        self.kestra_client.users.delete_user(id=created_user.id)

    def test_delete_user_auth_method(self) -> None:
        """Update user password (delete auth method) - use patch restricted as placeholder"""
        name_base = "test_delete_user_auth_method"
        user_req = IAMUserControllerApiCreateOrUpdateUserRequest(
            email=f"{name_base}@kestra.io",
            password="Password!234"
        )
        created_user = self.kestra_client.users.create_user(
            iam_user_controller_api_create_or_update_user_request=user_req
        )

        patched = self.kestra_client.users.delete_user_auth_method(
            id=created_user.id,
            auth=created_user.auths[0].id
        )
        assert getattr(patched, 'id', None) == created_user.id

    def test_get_user(self) -> None:
        """Get a user"""
        name_base = "test_get_user"
        user_req = IAMUserControllerApiCreateOrUpdateUserRequest(
            email=f"{name_base}@kestra.io"
        )
        created_user = self.kestra_client.users.create_user(
            iam_user_controller_api_create_or_update_user_request=user_req
        )

        fetched = self.kestra_client.users.user(id=created_user.id)
        assert fetched.id == created_user.id

        # cleanup
        try:
            self.kestra_client.users.delete_user(id=created_user.id)
        except Exception:
            pass

    def test_list_api_tokens_for_user(self) -> None:
        """Test case for list_api_tokens_for_user

        List API tokens for a specific user
        """
        name_base = "test_list_api_tokens_for_user"
        user_req = IAMUserControllerApiCreateOrUpdateUserRequest(
            email=f"{name_base}@kestra.io"
        )
        created_user = self.kestra_client.users.create_user(iam_user_controller_api_create_or_update_user_request=user_req)

        token_req = CreateApiTokenRequest(name=name_base.replace("_", "-"))
        created_token = self.kestra_client.users.create_api_tokens_for_user(id=created_user.id, create_api_token_request=token_req)

        tokens = self.kestra_client.users.list_api_tokens_for_user(id=created_user.id)
        assert any(getattr(t, 'id', None) == created_token.id for t in tokens.results)

    def test_list_users(self) -> None:
        """Retrieve users"""
        name_base = "test_list_users"
        # create a user
        user_req = IAMUserControllerApiCreateOrUpdateUserRequest(
            email=f"{name_base}@kestra.io"
        )
        created_user = self.kestra_client.users.create_user(iam_user_controller_api_create_or_update_user_request=user_req)

        results = self.kestra_client.users.list_users(page=1, size=50, q=name_base)
        assert any(getattr(r, 'id', None) == created_user.id for r in results.results)

    def test_list_users_fetch_all_pages(self) -> None:
        """Retrieve users without query"""
        name_base = "test_list_users_no_query"
        for i in range(60):
            user_req = IAMUserControllerApiCreateOrUpdateUserRequest(
                email = f"{name_base}_{i}@kestra.io"
            )
            self.kestra_client.users.create_user(iam_user_controller_api_create_or_update_user_request=user_req)

        users = []
        page = 1
        size = 25

        while True:
            results = self.kestra_client.users.list_users(page=page, size=size)
            users.extend(results.results)
            if len(results.results) < size:
                break
            page += 1

        assert len(users) >= 60

    def test_patch_user(self) -> None:
        """Update user details"""
        name_base = "test_patch_user"
        user_req = IAMUserControllerApiCreateOrUpdateUserRequest(
            email=f"{name_base}@kestra.io",
            first_name="Old"
        )
        created_user = self.kestra_client.users.create_user(iam_user_controller_api_create_or_update_user_request=user_req)

        update_req = IAMUserControllerApiCreateOrUpdateUserRequest(
            email=created_user.email,
            first_name="New"
        )
        updated = self.kestra_client.users.update_user(id=created_user.id, iam_user_controller_api_create_or_update_user_request=update_req)
        assert getattr(updated, 'first_name', None) == "New" or getattr(updated, 'firstName', None) == "New"

        # cleanup
        try:
            self.kestra_client.users.delete_user(id=created_user.id)
        except Exception:
            pass

    def test_patch_user_password(self) -> None:
        """Update user password"""
        name_base = "test_patch_user_password"
        user_req = IAMUserControllerApiCreateOrUpdateUserRequest(
            email=f"{name_base}@kestra.io",
            password="OldPass!1"
        )
        created_user = self.kestra_client.users.create_user(iam_user_controller_api_create_or_update_user_request=user_req)

        password_req = IAMUserControllerApiPatchUserPasswordRequest(password="NewPass!1")
        patched = self.kestra_client.users.patch_user_password(id=created_user.id, iam_user_controller_api_patch_user_password_request=password_req)
        assert getattr(patched, 'id', None) == created_user.id

    def test_patch_user_super_admin(self) -> None:
        """Update user superadmin privileges"""
        name_base = "test_patch_user_super_admin"
        user_req = IAMUserControllerApiCreateOrUpdateUserRequest(
            email=f"{name_base}@kestra.io"
        )
        created_user = self.kestra_client.users.create_user(iam_user_controller_api_create_or_update_user_request=user_req)

        patch = ApiPatchSuperAdminRequest(super_admin=True)
        self.kestra_client.users.patch_user_super_admin(id=created_user.id, api_patch_super_admin_request=patch)

        fetched = self.kestra_client.users.user(id=created_user.id)
        assert getattr(fetched, 'super_admin', None) is True

    def test_update_current_user_password(self) -> None:
        """Update authenticated user password for a dedicated test user (do not modify admin user)

        This test creates a new user with a known password, creates a client authenticated as that user,
        updates the user's password and then reverts it, and finally cleans up the created user.
        If creating the user is unauthorized, the test is skipped.
        """
        name_base = "test_update_current_user_password"
        email = f"{name_base}@kestra.io"
        initial_password = "InitialPass!1"
        changed_password = "ChangedPass!1"

        user_req = IAMUserControllerApiCreateOrUpdateUserRequest(email=email, password=initial_password, tenants=[self.tenant])
        created_user = self.kestra_client.users.create_user(iam_user_controller_api_create_or_update_user_request=user_req)

        # create a client authenticated as the test user
        user_conf = Configuration()
        user_conf.host = "http://localhost:9902"
        user_conf.username = email
        user_conf.password = initial_password
        user_client = KestraClient(user_conf)

        # change password using the dedicated client
        pwd_req = MeControllerApiUpdatePasswordRequest(old_password=initial_password, new_password=changed_password)
        user_client.users.update_current_user_password(me_controller_api_update_password_request=pwd_req)

        # update client and make sure he can change his password again using the new one
        user_conf.password = changed_password
        user_client = KestraClient(user_conf)

        # revert the password back
        revert_req = MeControllerApiUpdatePasswordRequest(old_password=changed_password, new_password=initial_password)
        user_client.users.update_current_user_password(me_controller_api_update_password_request=revert_req)

    def test_update_user(self) -> None:
        """Update a user account"""
        name_base = "test_update_user"
        user_req = IAMUserControllerApiCreateOrUpdateUserRequest(
            email=f"{name_base}@kestra.io",
            first_name="Before"
        )
        created_user = self.kestra_client.users.create_user(iam_user_controller_api_create_or_update_user_request=user_req)

        update_req = IAMUserControllerApiCreateOrUpdateUserRequest(
            email=created_user.email,
            first_name="After"
        )
        updated = self.kestra_client.users.update_user(id=created_user.id, iam_user_controller_api_create_or_update_user_request=update_req)
        assert getattr(updated, 'first_name', None) == "After" or getattr(updated, 'firstName', None) == "After"

    def test_update_user_groups(self) -> None:
        """Update the list of groups a user belongs to for the given tenant"""

        group = IAMGroupControllerApiCreateGroupRequest(
            name="test_update_user_groups",
            description="An example group"
        )

        created_group = self.kestra_client.groups.create_group(
            tenant=self.tenant,
            iam_group_controller_api_create_group_request=group
        )

        name_base = "test_update_user_groups"
        user_req = IAMUserControllerApiCreateOrUpdateUserRequest(
            email=f"{name_base}@kestra.io",
            tenants=[self.tenant]
        )
        created_user = self.kestra_client.users.create_user(iam_user_controller_api_create_or_update_user_request=user_req)

        groups_req = IAMUserGroupControllerApiUpdateUserGroupsRequest(group_ids=[created_group.id])
        self.kestra_client.users.update_user_groups(tenant=self.tenant, id=created_user.id, iam_user_group_controller_api_update_user_groups_request=groups_req)



if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Kestra EE

    All API operations, except for Superadmin-only endpoints, require a tenant identifier in the HTTP path.<br/> Endpoints designated as Superadmin-only are not tenant-scoped.

    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class ResourceType(str, Enum):
    """
    ResourceType
    """

    """
    allowed enum values
    """
    AUDITLOG = 'AUDITLOG'
    APP = 'APP'
    ASSET = 'ASSET'
    APP_EXECUTION = 'APP_EXECUTION'
    BLUEPRINT = 'BLUEPRINT'
    BINDING = 'BINDING'
    DASHBOARD = 'DASHBOARD'
    EXECUTION = 'EXECUTION'
    FLOW = 'FLOW'
    KV_STORE = 'KV_STORE'
    GROUP = 'GROUP'
    INVITATION = 'INVITATION'
    NAMESPACE = 'NAMESPACE'
    NAMESPACE_FILE = 'NAMESPACE_FILE'
    ROLE = 'ROLE'
    SECURITY_INTEGRATION = 'SECURITY_INTEGRATION'
    SECRET = 'SECRET'
    SETTING = 'SETTING'
    TEMPLATE = 'TEMPLATE'
    TRIGGER = 'TRIGGER'
    TENANT = 'TENANT'
    TENANT_ACCESS = 'TENANT_ACCESS'
    TESTSUITE = 'TESTSUITE'
    TESTSUITE_RUN = 'TESTSUITE_RUN'
    VERSIONED_PLUGIN = 'VERSIONED_PLUGIN'
    USER = 'USER'
    WORKER_GROUP = 'WORKER_GROUP'
    INSTANCE = 'INSTANCE'
    UNKNOWN = 'UNKNOWN'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ResourceType from a JSON string"""
        return cls(json.loads(json_str))






# coding: utf-8

"""
    Kestra EE

    All API operations, except for Superadmin-only endpoints, require a tenant identifier in the HTTP path.<br/> Endpoints designated as Superadmin-only are not tenant-scoped.

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from kestrapy import Configuration, KestraClient, State, ExecutionControllerStateRequest
from typing import Optional
from kestrapy import QueryFilter, QueryFilterField, QueryFilterOp, ExecutionControllerSetLabelsByIdsRequest, Label, StateType
import time


class TestExecutionsApi(unittest.TestCase):
    """ExecutionsApi unit test stubs"""

    def setUp(self) -> None:
        configuration = Configuration()
        configuration.host = "http://localhost:9902"
        configuration.username = "root@root.com"
        configuration.password = "Root!1234"

        self.kestra_client = KestraClient(configuration)
        self.tenant = "main"

    def tearDown(self) -> None:
        pass

    def create_flow(self, flow_id: Optional[str] = None, namespace: str = "test.executions") -> str:
        """Helper to create a simple flow from a YAML source and return its id."""
        if flow_id is None:
            flow_id = f"{self._testMethodName}"

        body = f"""id: {flow_id}
namespace: {namespace}

inputs:
  - id: inp
    type: STRING
    defaults: 'hello'

tasks:
  - id: hello
    type: io.kestra.plugin.core.log.Log
    message: "hello {{inputs.inp}}"
"""
        created = self.kestra_client.flows.create_flow(tenant=self.tenant, body=body)
        return getattr(created, 'id', flow_id)

    def create_flow_with_file(self, flow_id: Optional[str] = None, namespace: str = "test.executions") -> str:
        """Helper to create a flow that writes a file and returns its id.
        Resource name follows the test method name if flow_id is None.
        """
        if flow_id is None:
            flow_id = f"{self._testMethodName}"

        body = f"""id: {flow_id}
namespace: {namespace}

tasks:
  - id: write
    type: io.kestra.plugin.core.storage.Write
    content: "Hello from file"
    extension: .txt
"""
        created = self.kestra_client.flows.create_flow(tenant=self.tenant, body=body)
        return getattr(created, 'id', flow_id)

    def create_flow_sleeping(self, flow_id: Optional[str] = None, namespace: str = "test.executions", sleep_duration: str = "PT10S") -> str:
        """Helper to create a flow that sleeps and returns its id.
        Resource name follows the test method name if flow_id is None.
        """
        if flow_id is None:
            flow_id = f"{self._testMethodName}"

        body = f"""id: {flow_id}
namespace: {namespace}

tasks:
  - id: sleep
    type: io.kestra.plugin.core.flow.Sleep
    duration: {sleep_duration}
"""
        created = self.kestra_client.flows.create_flow(tenant=self.tenant, body=body)
        return getattr(created, 'id', flow_id)

    def create_flow_failing(self, flow_id: Optional[str] = None, namespace: str = "test.executions") -> str:
        """Helper to create a flow that fails and returns its id.
        Resource name follows the test method name if flow_id is None.
        """
        if flow_id is None:
            flow_id = f"{self._testMethodName}"

        body = f"""id: {flow_id}
inputs:
  - id: key
    type: STRING
    defaults: 'value'
namespace: {namespace}

tasks:
  - id: fail
    type: io.kestra.plugin.core.execution.Fail
"""
        created = self.kestra_client.flows.create_flow(tenant=self.tenant, body=body)
        return getattr(created, 'id', flow_id)

    def create_flow_pausing(self, flow_id: Optional[str] = None, namespace: str = "test.executions") -> str:
        """Helper to create a flow that pauses and returns its id.
        Resource name follows the test method name if flow_id is None.
        """
        if flow_id is None:
            flow_id = f"{self._testMethodName}"

        body = f"""id: {flow_id}
namespace: {namespace}

tasks:
  - id: pause
    type: io.kestra.plugin.core.flow.Pause
"""
        created = self.kestra_client.flows.create_flow(tenant=self.tenant, body=body)
        return getattr(created, 'id', flow_id)

    def create_flow_webhook(self, flow_id: str, namespace: str):
        """
        Create a flow with a Webhook trigger (key: a-secret-key) and a Return task
        that returns the trigger payload. Mirrors the Java `WEBHOOK_FLOW`.
        """
        flow = f"""id: {flow_id}
namespace: {namespace}

tasks:
  - id: out
    type: io.kestra.plugin.core.debug.Return
    format: "{{{{trigger | json }}}}"

triggers:
  - id: webhook
    type: io.kestra.plugin.core.trigger.Webhook
    key: a-secret-key
"""
        created = self.kestra_client.flows.create_flow(self.tenant, flow)
        return created

    def test_create_execution(self) -> None:
        """Test case for create_execution

        Create a new execution for a flow
        """
        namespace = f"test_create_execution_ns"
        flow_id = f"{self._testMethodName}_flow"
        self.create_flow(flow_id=flow_id, namespace=namespace)

        resp = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant, additional_form_datas={"inp": "override"})
        assert resp is not None
        exec_id = getattr(resp, 'id', None)
        assert exec_id is not None

    def test_delete_execution(self) -> None:
        """Test case for delete_execution

        Delete an execution
        """
        namespace = f"test_delete_execution_ns"
        flow_id = f"{self._testMethodName}_flow"
        self.create_flow(flow_id=flow_id, namespace=namespace)

        created = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        exec_id = getattr(created, 'id', None)
        assert exec_id is not None

        self.kestra_client.executions.delete_execution(execution_id=exec_id, tenant=self.tenant)

        with self.assertRaises(Exception):
            self.kestra_client.executions.execution(id=exec_id, tenant=self.tenant)

    def test_delete_executions_by_ids(self) -> None:
        """Test case for delete_executions_by_ids

        Delete a list of executions
        """
        namespace = f"test_delete_executions_by_ids_ns"
        f1 = f"{self._testMethodName}_1"
        f2 = f"{self._testMethodName}_2"
        self.create_flow(flow_id=f1, namespace=namespace)
        self.create_flow(flow_id=f2, namespace=namespace)

        exec_flow1 = self.kestra_client.executions.create_execution(namespace=namespace, id=f1, wait=False, tenant=self.tenant)
        exec_flow2 = self.kestra_client.executions.create_execution(namespace=namespace, id=f2, wait=False, tenant=self.tenant)
        id1 = getattr(exec_flow1, 'id', None)
        id2 = getattr(exec_flow2, 'id', None)
        assert id1 is not None
        assert id2 is not None

        resp = self.kestra_client.executions.delete_executions_by_ids(tenant=self.tenant, request_body=[id1, id2], include_non_terminated=True)
        assert resp is not None

        with self.assertRaises(Exception):
            self.kestra_client.executions.execution(id=id1, tenant=self.tenant)
        with self.assertRaises(Exception):
            self.kestra_client.executions.execution(id=id2, tenant=self.tenant)

    def test_delete_executions_by_query(self) -> None:
        """Test case for delete_executions_by_query

        Delete executions filter by query parameters
        """
        base = f"{self._testMethodName}"
        namespace = f"test_delete_executions_by_query_ns"
        flow_id = f"{base}_flow"
        self.create_flow(flow_id=flow_id, namespace=namespace)

        created = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        exec_id = getattr(created, 'id', None)
        assert exec_id is not None

        qf = QueryFilter(field=QueryFilterField.NAMESPACE, operation=QueryFilterOp.CONTAINS, value={"value": namespace})
        resp = self.kestra_client.executions.delete_executions_by_query(tenant=self.tenant, filters=[qf], include_non_terminated=True)
        assert resp is not None

        with self.assertRaises(Exception):
            self.kestra_client.executions.execution(id=exec_id, tenant=self.tenant)

    def test_download_file_from_execution(self) -> None:
        """Test case for download_file_from_execution

        Download file for an execution
        """
        namespace = f"test_download_file_from_execution_ns"
        flow_id = "test_download_file_from_execution"
        self.create_flow_with_file(flow_id=flow_id, namespace=namespace)

        created = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        exec_id = getattr(created, 'id', None)
        assert exec_id is not None

        time.sleep(2)

        execution = self.kestra_client.executions.execution(execution_id=exec_id, tenant=self.tenant)

        task_runs = getattr(execution, 'task_run_list', None) or []
        assert len(task_runs) > 0
        first = task_runs[0]
        outputs = getattr(first, 'outputs', {}) or {}
        uri = outputs.get('uri') if isinstance(outputs, dict) else None
        assert uri is not None

        resp = self.kestra_client.executions.download_file_from_execution(execution_id=exec_id, path=uri, tenant=self.tenant)
        content = None
        if isinstance(resp, (bytes, bytearray)):
            content = resp.decode('utf-8')

        assert "Hello from file" in content


    def test_follow_dependencies_executions(self) -> None:
        """Test case for follow_dependencies_executions

        Follow all execution dependencies executions
        """
        namespace = f"test_follow_dependencies_executions_ns"
        flow_id = f"{self._testMethodName}_flow"
        self.create_flow(flow_id=flow_id, namespace=namespace)

        created = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        exec_id = getattr(created, 'id', None)
        assert exec_id is not None

        following = self.kestra_client.executions.follow_dependencies_executions(execution_id=exec_id, tenant=self.tenant)

        got_event = False
        for event in following:
            assert event is not None
            got_event = True
            break

        assert got_event

    def test_follow_execution(self) -> None:
        """Test case for follow_execution

        Follow an execution
        """
        namespace = f"test_follow_execution_ns"
        flow_id = f"{self._testMethodName}_flow"
        self.create_flow(flow_id=flow_id, namespace=namespace)

        created = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        exec_id = getattr(created, 'id', None)
        assert exec_id is not None

        following = self.kestra_client.executions.follow_execution(execution_id=exec_id, tenant=self.tenant)

        got_event = False
        for event in following:
            assert event is not None
            got_event = True
            break

        assert got_event

    def test_force_run_by_ids(self) -> None:
        """Test case for force_run_by_ids

        Force run a list of executions
        """
        namespace = f"test_force_run_by_ids_ns"
        flow_id = f"{self._testMethodName}_flow"

        body = f"""id: {flow_id}
namespace: {namespace}

concurrency:
  behavior: QUEUE
  limit: 1

tasks:
  - id: sleep
    type: io.kestra.plugin.core.flow.Sleep
    duration: PT2S
"""
        self.kestra_client.flows.create_flow(tenant=self.tenant, body=body)

        first = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        first_id = getattr(first, 'id', None)
        assert first_id is not None

        for _ in range(10):
            st = self.kestra_client.executions.execution(execution_id=first_id, tenant=self.tenant)
            if getattr(getattr(st, 'state', None), 'current', None) == 'RUNNING':
                break
            time.sleep(0.2)

        queued = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        queued_id = getattr(queued, 'id', None)
        assert queued_id is not None

        for _ in range(10):
            st = self.kestra_client.executions.execution(execution_id=queued_id, tenant=self.tenant)
            if getattr(getattr(st, 'state', None), 'current', None) == 'QUEUED':
                break
            time.sleep(0.2)

        resp = self.kestra_client.executions.force_run_by_ids(tenant=self.tenant, request_body=[queued_id])
        assert resp is not None

        for _ in range(10):
            st = self.kestra_client.executions.execution(execution_id=queued_id, tenant=self.tenant)
            if getattr(getattr(st, 'state', None), 'current', None) == 'RUNNING':
                break
            time.sleep(0.2)
        assert getattr(getattr(self.kestra_client.executions.execution(execution_id=queued_id, tenant=self.tenant), 'state', None), 'current', None) == 'RUNNING'

    def test_force_run_execution(self) -> None:
        """Test case for force_run_execution

        Force run an execution
        """
        namespace = f"test_force_run_execution_ns"
        flow_id = f"{self._testMethodName}_flow"

        body = f"""id: {flow_id}
namespace: {namespace}

concurrency:
  behavior: QUEUE
  limit: 1

tasks:
  - id: sleep
    type: io.kestra.plugin.core.flow.Sleep
    duration: PT2S
"""
        self.kestra_client.flows.create_flow(tenant=self.tenant, body=body)

        first = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        first_id = getattr(first, 'id', None)
        assert first_id is not None

        for _ in range(10):
            st = self.kestra_client.executions.execution(execution_id=first_id, tenant=self.tenant)
            if getattr(getattr(st, 'state', None), 'current', None) == 'RUNNING':
                break
            time.sleep(0.2)

        queued = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        queued_id = getattr(queued, 'id', None)
        assert queued_id is not None

        for _ in range(10):
            st = self.kestra_client.executions.execution(execution_id=queued_id, tenant=self.tenant)
            if getattr(getattr(st, 'state', None), 'current', None) == 'QUEUED':
                break
            time.sleep(0.2)

        resp = self.kestra_client.executions.force_run_execution(execution_id=queued_id, tenant=self.tenant)
        assert resp is not None

        for _ in range(10):
            st = self.kestra_client.executions.execution(execution_id=queued_id, tenant=self.tenant)
            if getattr(getattr(st, 'state', None), 'current', None) == 'RUNNING':
                break
            time.sleep(0.2)
        assert getattr(getattr(self.kestra_client.executions.execution(execution_id=queued_id, tenant=self.tenant), 'state', None), 'current', None) == 'RUNNING'

    def test_force_run_executions_by_query(self) -> None:
        """Test case for force_run_executions_by_query

        Force run executions filter by query parameters
        """
        namespace = f"test_force_run_executions_by_query_ns"
        flow_id = f"{self._testMethodName}_flow"

        body = f"""id: {flow_id}
namespace: {namespace}

concurrency:
  behavior: QUEUE
  limit: 1

tasks:
  - id: sleep
    type: io.kestra.plugin.core.flow.Sleep
    duration: PT2S
    """
        self.kestra_client.flows.create_flow(tenant=self.tenant, body=body)

        first = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        first_id = getattr(first, 'id', None)
        assert first_id is not None

        for _ in range(10):
            st = self.kestra_client.executions.execution(execution_id=first_id, tenant=self.tenant)
            if getattr(getattr(st, 'state', None), 'current', None) == 'RUNNING':
                break
            time.sleep(0.2)

        queued = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        queued_id = getattr(queued, 'id', None)
        assert queued_id is not None

        for _ in range(10):
            st = self.kestra_client.executions.execution(execution_id=queued_id, tenant=self.tenant)
            if getattr(getattr(st, 'state', None), 'current', None) == 'QUEUED':
                break
            time.sleep(0.2)

        qf = QueryFilter(field=QueryFilterField.NAMESPACE, operation=QueryFilterOp.CONTAINS, value={"value": namespace})
        resp = self.kestra_client.executions.force_run_executions_by_query(tenant=self.tenant, filters=[qf])
        assert resp is not None

    def test_get_execution(self) -> None:
        """Test case for get_execution

        Get an execution
        """
        namespace = f"test_get_execution_ns"
        flow_id = f"{self._testMethodName}_flow"
        self.create_flow(flow_id=flow_id, namespace=namespace)

        created = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        exec_id = getattr(created, 'id', None)
        assert exec_id is not None

        resp = self.kestra_client.executions.execution(execution_id=exec_id, tenant=self.tenant)
        assert resp is not None
        assert getattr(resp, 'id', None) == exec_id

    def test_get_execution_flow_graph(self) -> None:
        """Test case for get_execution_flow_graph

        Generate a graph for an execution
        """
        namespace = f"test_get_execution_flow_graph_ns"
        flow_id = f"{self._testMethodName}_flow"
        self.create_flow(flow_id=flow_id, namespace=namespace)

        created = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        exec_id = getattr(created, 'id', None)
        assert exec_id is not None

        time.sleep(0.5)

        resp = self.kestra_client.executions.execution_flow_graph(execution_id=exec_id, tenant=self.tenant, subflows=None)
        assert resp is not None

    def test_get_file_metadatas_from_execution(self) -> None:
        """Test case for get_file_metadatas_from_execution

        Get file meta information for an execution
        """
        namespace = f"test_get_file_metadatas_from_execution_ns"
        flow_id = f"{self._testMethodName}_flow"

        self.create_flow_with_file(flow_id=flow_id, namespace=namespace)
        exec_resp = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        exec_id = getattr(exec_resp, 'id', None)
        assert exec_id is not None

        for _ in range(50):
            st = self.kestra_client.executions.execution(execution_id=exec_id, tenant=self.tenant)
            if getattr(getattr(st, 'state', None), 'current', None) == 'SUCCESS':
                break
            time.sleep(0.2)

        execution = self.kestra_client.executions.execution(execution_id=exec_id, tenant=self.tenant)
        path = None
        for tr in getattr(execution, 'task_run_list', []) or []:
            outputs = getattr(tr, 'outputs', None) or {}
            if 'uri' in outputs:
                path = outputs['uri']
                break
        assert path is not None

        resp = self.kestra_client.executions.file_metadatas_from_execution(execution_id=exec_id, path=path, tenant=self.tenant)
        assert getattr(resp, 'size', None) == 15

    def test_get_flow_from_execution(self) -> None:
        """Test case for get_flow_from_execution

        Get flow information's for an execution
        """
        namespace = f"test_get_flow_from_execution_ns"
        flow_id = f"{self._testMethodName}_flow"

        self.create_flow(flow_id=flow_id, namespace=namespace)
        exec_resp = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        exec_id = getattr(exec_resp, 'id', None)
        assert exec_id is not None

        for _ in range(50):
            st = self.kestra_client.executions.execution(execution_id=exec_id, tenant=self.tenant)
            if getattr(getattr(st, 'state', None), 'current', None) == 'SUCCESS':
                break
            time.sleep(0.2)

        resp = self.kestra_client.executions.flow_from_execution(flow_id=flow_id, namespace=namespace, tenant=self.tenant)
        assert getattr(resp, 'id', None) == flow_id
        assert getattr(resp, 'namespace', None) == namespace

    def test_get_flow_from_execution_by_id(self) -> None:
        """Test case for get_flow_from_execution_by_id

        Get flow information's for an execution
        """
        namespace = f"test_get_flow_from_execution_by_id_ns"
        flow_id = f"{self._testMethodName}_flow"

        self.create_flow(flow_id=flow_id, namespace=namespace)
        exec_resp = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        exec_id = getattr(exec_resp, 'id', None)
        assert exec_id is not None

        for _ in range(50):
            st = self.kestra_client.executions.execution(execution_id=exec_id, tenant=self.tenant)
            if getattr(getattr(st, 'state', None), 'current', None) == 'SUCCESS':
                break
            time.sleep(0.2)

        resp = self.kestra_client.executions.flow_from_execution_by_id(execution_id=exec_id, tenant=self.tenant)
        assert getattr(resp, 'id', None) == flow_id
        assert getattr(resp, 'namespace', None) == namespace

    def test_get_latest_executions(self) -> None:
        """Test case for get_latest_executions

        Get the latest execution for given flows
        """
        namespace = f"test_get_latest_executions_ns"
        flow_id = f"{self._testMethodName}_flow"
        other_flow_id = f"{self._testMethodName}_other_flow"

        self.create_flow(flow_id=flow_id, namespace=namespace)
        first = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)

        second = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        self.create_flow(flow_id=other_flow_id, namespace=namespace)
        other = self.kestra_client.executions.create_execution(namespace=namespace, id=other_flow_id, wait=False, tenant=self.tenant)

        time.sleep(0.2)

        request_body = [
            {"id": flow_id, "namespace": namespace},
            {"id": other_flow_id, "namespace": namespace},
        ]
        resp = self.kestra_client.executions.latest_executions(
            execution_repository_interface_flow_filter=request_body,
            tenant=self.tenant,
        )
        assert resp is not None
        try:
            length = len(resp)
        except Exception:
            length = getattr(resp, '__len__', lambda: 0)()
        assert length == 2


    
    def test_kill_execution(self) -> None:
        """Test case for kill_execution

        Kill an execution
        """
        namespace = f"test_kill_execution_ns"
        flow_id = f"{self._testMethodName}_flow"

        self.create_flow_sleeping(flow_id=flow_id, namespace=namespace, sleep_duration="PT30S")

        created = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        exec_id = getattr(created, 'id', None)
        assert exec_id is not None

        time.sleep(0.5)

        self.kestra_client.executions.kill_execution(execution_id=exec_id, tenant=self.tenant, is_on_kill_cascade=False)

        time.sleep(0.5)

        st = self.kestra_client.executions.execution(execution_id=exec_id, tenant=self.tenant)
        cur = getattr(getattr(st, 'state', None), 'current', None)
        assert cur in ('KILLED', 'FAILED', 'ABORTED', 'TERMINATED')

    def test_kill_executions_by_ids(self) -> None:
        """Test case for kill_executions_by_ids

        Kill a list of executions
        """
        namespace = f"test_kill_executions_by_ids_ns"
        f1 = f"{self._testMethodName}_1"
        f2 = f"{self._testMethodName}_2"


        self.create_flow_sleeping(flow_id=f1, namespace=namespace, sleep_duration="PT10S")

        self.create_flow_sleeping(flow_id=f2, namespace=namespace, sleep_duration="PT10S")

        exec_flow1 = self.kestra_client.executions.create_execution(namespace=namespace, id=f1, wait=False, tenant=self.tenant)
        exec_flow2 = self.kestra_client.executions.create_execution(namespace=namespace, id=f2, wait=False, tenant=self.tenant)

        id1 = getattr(exec_flow1, 'id', None)
        id2 = getattr(exec_flow2, 'id', None)
        assert id1 is not None
        assert id2 is not None

        time.sleep(0.5)

        self.kestra_client.executions.kill_executions_by_ids(tenant=self.tenant, request_body=[id1, id2])

        time.sleep(0.5)

        for target in (id1, id2):
            st = self.kestra_client.executions.execution(execution_id=target, tenant=self.tenant)
            cur = getattr(getattr(st, 'state', None), 'current', None)
            assert cur in ('KILLED', 'FAILED', 'ABORTED', 'TERMINATED')

    def test_kill_executions_by_query(self) -> None:
        """Test case for kill_executions_by_query

        Kill executions filter by query parameters
        """
        namespace = f"test_kill_executions_by_query_ns"
        flow_id = f"{self._testMethodName}_flow"

        self.create_flow_sleeping(flow_id=flow_id, namespace=namespace, sleep_duration="PT10S")

        created = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        exec_id = getattr(created, 'id', None)
        assert exec_id is not None

        time.sleep(0.5)

        qf = QueryFilter(field=QueryFilterField.NAMESPACE, operation=QueryFilterOp.CONTAINS, value={"value": namespace})
        self.kestra_client.executions.kill_executions_by_query(tenant=self.tenant, filters=[qf])

        time.sleep(0.5)

        st = self.kestra_client.executions.execution(execution_id=exec_id, tenant=self.tenant)
        cur = getattr(getattr(st, 'state', None), 'current', None)
        assert cur in ('KILLED', 'FAILED', 'ABORTED', 'TERMINATED')

    def test_pause_execution(self) -> None:
        """Test case for pause_execution

        Pause a running execution.
        """
        namespace = f"test_pause_execution_ns"
        flow_id = f"{self._testMethodName}_flow"

        self.create_flow_sleeping(flow_id=flow_id, namespace=namespace, sleep_duration="PT30S")

        created = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        exec_id = getattr(created, 'id', None)
        assert exec_id is not None

        time.sleep(0.2)

        self.kestra_client.executions.pause_execution(execution_id=exec_id, tenant=self.tenant)

        time.sleep(0.2)

        st = self.kestra_client.executions.execution(execution_id=exec_id, tenant=self.tenant)
        assert getattr(getattr(st, 'state', None), 'current', None) == 'PAUSED'

    def test_pause_executions_by_ids(self) -> None:
        """Test case for pause_executions_by_ids

        Pause a list of running executions
        """
        namespace = f"test_pause_executions_by_ids_ns"
        f1 = f"{self._testMethodName}_1"
        f2 = f"{self._testMethodName}_2"

        self.create_flow_sleeping(flow_id=f1, namespace=namespace, sleep_duration="PT10S")

        self.create_flow_sleeping(flow_id=f2, namespace=namespace, sleep_duration="PT10S")

        exec_flow1 = self.kestra_client.executions.create_execution(namespace=namespace, id=f1, wait=False, tenant=self.tenant)
        exec_flow2 = self.kestra_client.executions.create_execution(namespace=namespace, id=f2, wait=False, tenant=self.tenant)
        id1 = getattr(exec_flow1, 'id', None)
        id2 = getattr(exec_flow2, 'id', None)
        assert id1 is not None
        assert id2 is not None

        time.sleep(0.5)

        self.kestra_client.executions.pause_executions_by_ids(tenant=self.tenant, request_body=[id1, id2])

        for target in (id1, id2):
            time.sleep(0.5)
            st = self.kestra_client.executions.execution(execution_id=target, tenant=self.tenant)
            assert getattr(getattr(st, 'state', None), 'current', None) == 'PAUSED'

    def test_pause_executions_by_query(self) -> None:
        """Test case for pause_executions_by_query

        Pause executions filter by query parameters
        """
        namespace = f"test_pause_executions_by_query_ns"
        flow_id = f"{self._testMethodName}_flow"

        self.create_flow_sleeping(flow_id=flow_id, namespace=namespace, sleep_duration="PT10S")

        created = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        exec_id = getattr(created, 'id', None)
        assert exec_id is not None

        time.sleep(0.5)

        qf = QueryFilter(field=QueryFilterField.NAMESPACE, operation=QueryFilterOp.CONTAINS, value={"value": namespace})
        resp = self.kestra_client.executions.pause_executions_by_query(tenant=self.tenant, filters=[qf])
        assert resp is not None

        time.sleep(0.5)
        st = self.kestra_client.executions.execution(execution_id=exec_id, tenant=self.tenant)
        assert getattr(getattr(st, 'state', None), 'current', None) == 'PAUSED'

    def test_replay_execution(self) -> None:
        """Test case for replay_execution

        Create a new execution from an old one and start it from a specified task run id
        """
        namespace = f"test_replay_execution_ns"
        flow_id = f"{self._testMethodName}_flow"
        self.create_flow_failing(flow_id=flow_id, namespace=namespace)

        exec_resp = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        exec_id = getattr(exec_resp, 'id', None)
        assert exec_id is not None

        time.sleep(0.5)

        execution = self.kestra_client.executions.execution(execution_id=exec_id, tenant=self.tenant)
        task_run_id = None
        for tr in getattr(execution, 'task_run_list', []) or []:
            task_run_id = getattr(tr, 'id', None)
            if task_run_id:
                break
        assert task_run_id is not None

        resp = self.kestra_client.executions.replay_execution(execution_id=exec_id, tenant=self.tenant, task_run_id=task_run_id)
        assert resp is not None
        new_exec_id = getattr(resp, 'id', None)
        assert new_exec_id is not None


    def test_replay_execution_withinputs(self) -> None:
        """Test case for replay_execution_withinputs

        Create a new execution from an old one and start it from a specified task run id
        """
        namespace = f"test_replay_execution_withinputs_ns"
        flow_id = f"{self._testMethodName}_flow"
        self.create_flow_failing(flow_id=flow_id, namespace=namespace)

        exec_resp = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        exec_id = getattr(exec_resp, 'id', None)
        assert exec_id is not None

        time.sleep(0.5)

        execution = self.kestra_client.executions.execution(execution_id=exec_id, tenant=self.tenant)
        task_run_id = None
        for tr in getattr(execution, 'task_run_list', []) or []:
            task_run_id = getattr(tr, 'id', None)
            if task_run_id:
                break
        assert task_run_id is not None

        resp = self.kestra_client.executions.replay_execution_withinputs(
            execution_id=exec_id,
            tenant=self.tenant,
            task_run_id=task_run_id,
            additional_form_datas={"inp": "replayed"}
        )
        assert resp is not None
        new_exec_id = getattr(resp, 'id', None)
        assert new_exec_id is not None


    def test_replay_executions_by_ids(self) -> None:
        """Test case for replay_executions_by_ids

        Create new executions from old ones. Keep the flow revision
        """
        namespace = f"test_replay_executions_by_ids_ns"
        flow_id = f"{self._testMethodName}_flow"
        self.create_flow_failing(flow_id=flow_id, namespace=namespace)

        created = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        exec_id = getattr(created, 'id', None)
        assert exec_id is not None

        time.sleep(0.5)

        resp = self.kestra_client.executions.replay_executions_by_ids(tenant=self.tenant, request_body=[exec_id])
        assert resp is not None


    def test_replay_executions_by_query(self) -> None:
        """Test case for replay_executions_by_query

        Create new executions from old ones filter by query parameters. Keep the flow revision
        """
        namespace = f"test_replay_executions_by_query_ns"
        flow_id = f"{self._testMethodName}_flow"
        self.create_flow_failing(flow_id=flow_id, namespace=namespace)

        created = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        exec_id = getattr(created, 'id', None)
        assert exec_id is not None

        time.sleep(0.5)

        qf = QueryFilter(field=QueryFilterField.NAMESPACE, operation=QueryFilterOp.CONTAINS, value={"value": namespace})
        resp = self.kestra_client.executions.replay_executions_by_query(tenant=self.tenant, filters=[qf])
        assert resp is not None

    def test_restart_execution(self) -> None:
        """Test case for restart_execution

        Restart a new execution from an old one
        """
        namespace = f"test_restart_execution_ns"
        flow_id = f"{self._testMethodName}_flow"
        self.create_flow_failing(flow_id=flow_id, namespace=namespace)

        exec_resp = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        exec_id = getattr(exec_resp, 'id', None)
        assert exec_id is not None

        time.sleep(0.5)

        self.kestra_client.executions.execution(execution_id=exec_id, tenant=self.tenant)

        resp = self.kestra_client.executions.restart_execution(execution_id=exec_id, tenant=self.tenant)
        assert resp is not None
        new_exec_id = getattr(resp, 'id', None)
        assert new_exec_id is not None


    def test_restart_executions_by_ids(self) -> None:
        """Test case for restart_executions_by_ids

        Restart a list of executions
        """
        namespace = f"test_restart_executions_by_ids_ns"
        flow_id = f"{self._testMethodName}_flow"
        self.create_flow_failing(flow_id=flow_id, namespace=namespace)

        created = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        exec_id = getattr(created, 'id', None)
        assert exec_id is not None

        time.sleep(0.5)

        resp = self.kestra_client.executions.restart_executions_by_ids(tenant=self.tenant, request_body=[exec_id])
        assert resp is not None


    def test_restart_executions_by_query(self) -> None:
        """Test case for restart_executions_by_query

        Restart executions filter by query parameters.
        """
        namespace = f"test_restart_executions_by_query_ns"
        flow_id = f"{self._testMethodName}_flow"
        self.create_flow_failing(flow_id=flow_id, namespace=namespace)

        created = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        exec_id = getattr(created, 'id', None)
        assert exec_id is not None

        time.sleep(0.5)

        qf = QueryFilter(field=QueryFilterField.NAMESPACE, operation=QueryFilterOp.CONTAINS, value={"value": namespace})
        resp = self.kestra_client.executions.restart_executions_by_query(tenant=self.tenant, filters=[qf])
        assert resp is not None


    def test_resume_execution(self) -> None:
        """Test case for resume_execution

        Resume a paused execution.
        """
        namespace = f"test_resume_execution_ns"
        flow_id = f"{self._testMethodName}_flow"
        self.create_flow_pausing(flow_id=flow_id, namespace=namespace)

        created = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        exec_id = getattr(created, 'id', None)
        assert exec_id is not None

        time.sleep(0.5)

        self.kestra_client.executions.resume_execution(execution_id=exec_id, tenant=self.tenant)

        exec = self.kestra_client.executions.execution(execution_id=exec_id, tenant=self.tenant)
        cur = getattr(getattr(exec, 'state', None), 'current', None)
        assert cur not in ('PAUSED', 'BREAKPOINT')

    def test_resume_executions_by_ids(self) -> None:
        """Test case for resume_executions_by_ids

        Resume a list of paused executions
        """
        namespace = f"test_resume_executions_by_ids_ns"
        flow_id = f"{self._testMethodName}_flow"
        self.create_flow_pausing(flow_id=flow_id, namespace=namespace)

        created = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        exec_id = getattr(created, 'id', None)
        assert exec_id is not None

        time.sleep(0.5)

        resp = self.kestra_client.executions.resume_executions_by_ids(tenant=self.tenant, request_body=[exec_id])
        assert resp is not None

        time.sleep(0.5)

        st = self.kestra_client.executions.execution(execution_id=exec_id, tenant=self.tenant)
        cur = getattr(getattr(st, 'state', None), 'current', None)
        assert cur not in ('PAUSED', 'BREAKPOINT')


    def test_resume_executions_by_query(self) -> None:
        """Test case for resume_executions_by_query

        Resume executions filter by query parameters
        """
        namespace = f"test_resume_executions_by_query_ns"
        flow_id = f"{self._testMethodName}_flow"
        self.create_flow_pausing(flow_id=flow_id, namespace=namespace)

        created = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        exec_id = getattr(created, 'id', None)
        assert exec_id is not None


        time.sleep(0.5)

        qf = QueryFilter(field=QueryFilterField.NAMESPACE, operation=QueryFilterOp.CONTAINS, value={"value": namespace})
        resp = self.kestra_client.executions.resume_executions_by_query(tenant=self.tenant, filters=[qf])
        assert resp is not None

        time.sleep(0.5)

        st = self.kestra_client.executions.execution(execution_id=exec_id, tenant=self.tenant)
        cur = getattr(getattr(st, 'state', None), 'current', None)

        assert cur not in ('PAUSED', 'BREAKPOINT')


    def test_search_executions(self) -> None:
        """Test case for search_executions

        Search for executions
        """
        namespace = f"test_search_executions_ns"
        flow_id = f"{self._testMethodName}_flow"
        self.create_flow(flow_id=flow_id, namespace=namespace)

        created = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        exec_id = getattr(created, 'id', None)
        assert exec_id is not None

        time.sleep(0.5)

        qf = QueryFilter(field=QueryFilterField.NAMESPACE, operation=QueryFilterOp.CONTAINS, value={"value": namespace})
        resp = self.kestra_client.executions.search_executions(tenant=self.tenant, filters=[qf], page=1, size=10)
        assert resp is not None
        assert len(resp.results) > 0


    def test_search_executions_by_flow_id(self) -> None:
        """Test case for search_executions_by_flow_id

        Search for executions for a flow
        """
        namespace = f"test_search_executions_by_flow_id_ns"
        flow_id = f"{self._testMethodName}_flow"
        self.create_flow(flow_id=flow_id, namespace=namespace)

        created = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        exec_id = getattr(created, 'id', None)
        assert exec_id is not None

        time.sleep(0.5)

        resp = self.kestra_client.executions.search_executions_by_flow_id(flow_id=flow_id, namespace=namespace, tenant=self.tenant, page=1, size=10)
        assert resp is not None
        assert len(resp.results) > 0


    def test_set_labels_on_terminated_execution(self) -> None:
        """Test case for set_labels_on_terminated_execution

        Add or update labels of a terminated execution
        """
        namespace = f"test_set_labels_on_terminated_execution_ns"
        flow_id = f"{self._testMethodName}_flow"
        self.create_flow(flow_id=flow_id, namespace=namespace)

        created = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        exec_id = getattr(created, 'id', None)
        assert exec_id is not None

        time.sleep(0.5)

        resp = self.kestra_client.executions.set_labels_on_terminated_execution(
            execution_id=exec_id,
            tenant=self.tenant,
            label=[Label(key="test_set_labels_on_terminated_execution", value="yes")]
        )
        assert resp is not None


    def test_set_labels_on_terminated_executions_by_ids(self) -> None:
        """Test case for set_labels_on_terminated_executions_by_ids

        Set labels on a list of executions
        """
        namespace = f"test_set_labels_on_terminated_executions_by_ids_ns"
        flow_id = f"{self._testMethodName}_flow"
        self.create_flow(flow_id=flow_id, namespace=namespace)

        created = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        exec_id = getattr(created, 'id', None)
        assert exec_id is not None

        time.sleep(0.5)

        body = ExecutionControllerSetLabelsByIdsRequest(
            executions_id=[exec_id],
            execution_labels=[Label(key="test_set_labels_on_terminated_executions_by_ids", value="yes")]
        )

        resp = self.kestra_client.executions.set_labels_on_terminated_executions_by_ids(
            tenant=self.tenant,
            execution_controller_set_labels_by_ids_request=body
        )
        assert resp is not None


    def test_set_labels_on_terminated_executions_by_query(self) -> None:
        """Test ca\ase for set_labels_on_terminated_executions_by_query

        Set label on executions filter by query parameters.
        """
        namespace = f"test_set_labels_on_terminated_executions_by_query_ns"
        flow_id = f"{self._testMethodName}_flow"
        self.create_flow(flow_id=flow_id, namespace=namespace)

        created = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        exec_id = getattr(created, 'id', None)
        assert exec_id is not None

        time.sleep(0.5)

        qf = QueryFilter(field=QueryFilterField.NAMESPACE, operation=QueryFilterOp.CONTAINS, value={"value": namespace})
        resp = self.kestra_client.executions.set_labels_on_terminated_executions_by_query(
            tenant=self.tenant,
            filters=[qf],
            label=[Label(key="test_set_labels_on_terminated_executions_by_query", value="yes")]
        )
        assert resp is not None

    def test_trigger_execution_by_get_webhook(self) -> None:
        """Test case for trigger_execution_by_get_webhook

        Trigger a new execution by GET webhook trigger using the webhook flow helper
        """
        namespace = f"test_trigger_execution_by_get_webhook_ns"
        flow_id = f"{self._testMethodName}_flow"
        # create the webhook flow using the helper
        self.create_flow_webhook(flow_id=flow_id, namespace=namespace)

        key = "a-secret-key"
        resp = self.kestra_client.executions.trigger_execution_by_get_webhook(namespace=namespace, id=flow_id, key=key, tenant=self.tenant)
        assert resp is not None

        exec_id = getattr(resp, "id", None)
        assert exec_id is not None

        time.sleep(0.5)

        st = self.kestra_client.executions.execution(execution_id=exec_id, tenant=self.tenant)
        cur = getattr(getattr(st, "state", None), "current", None)
        assert cur == "SUCCESS"

    def test_unqueue_execution(self) -> None:
        """Test case for unqueue_execution

        Unqueue an execution
        """
        namespace = f"test_unqueue_execution_ns"
        flow_id = f"{self._testMethodName}_flow"

        body = f"""id: {flow_id}
namespace: {namespace}

concurrency:
  behavior: QUEUE
  limit: 1

tasks:
  - id: sleep
    type: io.kestra.plugin.core.flow.Sleep
    duration: PT2S
"""
        self.kestra_client.flows.create_flow(tenant=self.tenant, body=body)

        first = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        first_id = getattr(first, 'id', None)
        assert first_id is not None

        for _ in range(10):
            st = self.kestra_client.executions.execution(execution_id=first_id, tenant=self.tenant)
            if getattr(getattr(st, 'state', None), 'current', None) == 'RUNNING':
                break
            time.sleep(0.2)

        queued = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        queued_id = getattr(queued, 'id', None)
        assert queued_id is not None

        for _ in range(10):
            st = self.kestra_client.executions.execution(execution_id=queued_id, tenant=self.tenant)
            if getattr(getattr(st, 'state', None), 'current', None) == 'QUEUED':
                break
            time.sleep(0.2)

        resp = self.kestra_client.executions.unqueue_execution(execution_id=queued_id, state='RUNNING', tenant=self.tenant)
        assert resp is not None

        for _ in range(10):
            st = self.kestra_client.executions.execution(execution_id=queued_id, tenant=self.tenant)
            if getattr(getattr(st, 'state', None), 'current', None) == 'RUNNING':
                break
            time.sleep(0.2)
        assert getattr(getattr(self.kestra_client.executions.execution(execution_id=queued_id, tenant=self.tenant), 'state', None), 'current', None) == 'RUNNING'


    def test_unqueue_executions_by_ids(self) -> None:
        """Test case for unqueue_executions_by_ids

        Unqueue a list of executions
        """
        namespace = f"test_unqueue_executions_by_ids_ns"
        flow_id = f"{self._testMethodName}_flow"

        body = f"""id: {flow_id}
namespace: {namespace}

concurrency:
  behavior: QUEUE
  limit: 1

tasks:
  - id: sleep
    type: io.kestra.plugin.core.flow.Sleep
    duration: PT2S
"""
        self.kestra_client.flows.create_flow(tenant=self.tenant, body=body)

        first = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        first_id = getattr(first, 'id', None)
        assert first_id is not None

        for _ in range(10):
            st = self.kestra_client.executions.execution(execution_id=first_id, tenant=self.tenant)
            if getattr(getattr(st, 'state', None), 'current', None) == 'RUNNING':
                break
            time.sleep(0.2)

        queued1 = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        queued1_id = getattr(queued1, 'id', None)
        assert queued1_id is not None

        queued2 = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        queued2_id = getattr(queued2, 'id', None)
        assert queued2_id is not None

        time.sleep(0.5)

        resp = self.kestra_client.executions.unqueue_executions_by_ids(state='RUNNING', tenant=self.tenant, request_body=[queued1_id, queued2_id])
        assert resp is not None

        time.sleep(0.5)

        assert getattr(getattr(self.kestra_client.executions.execution(execution_id=queued1_id, tenant=self.tenant), 'state', None), 'current', None) == 'RUNNING'
        assert getattr(getattr(self.kestra_client.executions.execution(execution_id=queued2_id, tenant=self.tenant), 'state', None), 'current', None) == 'RUNNING'


    def test_unqueue_executions_by_query(self) -> None:
        """Test case for unqueue_executions_by_query

        Unqueue executions filter by query parameters
        """
        namespace = f"test_unqueue_executions_by_query_ns"
        flow_id = f"{self._testMethodName}_flow"

        body = f"""id: {flow_id}
namespace: {namespace}

concurrency:
  behavior: QUEUE
  limit: 1

tasks:
  - id: sleep
    type: io.kestra.plugin.core.flow.Sleep
    duration: PT30S
"""
        self.kestra_client.flows.create_flow(tenant=self.tenant, body=body)

        first = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        first_id = getattr(first, 'id', None)
        assert first_id is not None

        time.sleep(0.5)

        queued1 = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        queued1_id = getattr(queued1, 'id', None)
        assert queued1_id is not None

        queued2 = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        queued2_id = getattr(queued2, 'id', None)
        assert queued2_id is not None

        time.sleep(0.5)

        filters = [QueryFilter(field=QueryFilterField.QUERY, operation=QueryFilterOp.EQUALS, value={"value": queued1_id})]

        resp = self.kestra_client.executions.unqueue_executions_by_query(tenant=self.tenant, filters=filters, new_state=StateType.RUNNING)
        assert resp is not None

        time.sleep(0.5)

        assert getattr(getattr(self.kestra_client.executions.execution(execution_id=queued1_id, tenant=self.tenant), 'state', None), 'current', None) == 'RUNNING'


    def test_update_execution_status(self) -> None:
        """Test case for update_execution_status

        Change the state of an execution
        """
        namespace = f"test_update_execution_status_ns"
        flow_id = f"{self._testMethodName}_flow"

        body = f"""id: {flow_id}
namespace: {namespace}

tasks:
  - id: hello
    type: io.kestra.plugin.core.log.Log
    message: Hello World!
"""
        self.kestra_client.flows.create_flow(tenant=self.tenant, body=body)

        resp = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        exec_id = getattr(resp, "id", None)
        assert exec_id is not None

        time.sleep(0.5)

        updated = self.kestra_client.executions.update_execution_status(execution_id=exec_id, status="CANCELLED", tenant=self.tenant)
        assert updated is not None
        assert getattr(getattr(updated, "state", None), "current", None) == "CANCELLED"

        st = self.kestra_client.executions.execution(execution_id=exec_id, tenant=self.tenant)
        assert getattr(getattr(st, "state", None), "current", None) == "CANCELLED"


    def test_update_executions_status_by_ids(self) -> None:
        """Test case for update_executions_status_by_ids

        Change executions state by id
        """
        namespace = f"test_update_executions_status_by_ids_ns"
        flow_id = f"{self._testMethodName}_flow"

        body = f"""id: {flow_id}
namespace: {namespace}

tasks:
  - id: hello
    type: io.kestra.plugin.core.log.Log
    message: Hello World!
"""
        self.kestra_client.flows.create_flow(tenant=self.tenant, body=body)

        exec1 = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        exec1_id = getattr(exec1, "id", None)
        assert exec1_id is not None

        time.sleep(0.5)

        exec2 = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        exec2_id = getattr(exec2, "id", None)
        assert exec2_id is not None

        other = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        other_id = getattr(other, "id", None)
        assert other_id is not None

        time.sleep(0.5)

        resp = self.kestra_client.executions.update_executions_status_by_ids(new_status="CANCELLED", tenant=self.tenant, request_body=[exec2_id, other_id])
        assert resp is not None

        time.sleep(0.5)

        assert getattr(getattr(self.kestra_client.executions.execution(execution_id=exec2_id, tenant=self.tenant), "state", None), "current", None) == "CANCELLED"
        assert getattr(getattr(self.kestra_client.executions.execution(execution_id=other_id, tenant=self.tenant), "state", None), "current", None) == "CANCELLED"


    def test_update_executions_status_by_query(self) -> None:
        """Test case for update_executions_status_by_query

        Change executions state by query parameters
        """
        namespace = f"test_update_executions_status_by_query_ns"
        flow_id = f"{self._testMethodName}_flow"

        body = f"""id: {flow_id}
namespace: {namespace}

tasks:
  - id: hello
    type: io.kestra.plugin.core.log.Log
    message: Hello World!
"""
        self.kestra_client.flows.create_flow(tenant=self.tenant, body=body)

        exec1 = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        exec1_id = getattr(exec1, "id", None)
        assert exec1_id is not None

        time.sleep(0.5)

        exec2 = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        exec2_id = getattr(exec2, "id", None)
        assert exec2_id is not None

        other = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=False, tenant=self.tenant)
        other_id = getattr(other, "id", None)
        assert other_id is not None

        time.sleep(0.5)

        filters = [QueryFilter(field=QueryFilterField.FLOW_ID, operation=QueryFilterOp.EQUALS, value={"value": flow_id})]
        resp = self.kestra_client.executions.update_executions_status_by_query(new_status="CANCELLED", tenant=self.tenant, filters=filters)
        assert resp is not None

        time.sleep(0.5)

        assert getattr(getattr(self.kestra_client.executions.execution(execution_id=exec2_id, tenant=self.tenant), "state", None), "current", None) == "CANCELLED"

    def test_update_task_run_state(self) -> None:
        namespace = f"test.update_task_state_{int(time.time() * 1000)}"
        flow_id = f"flow-{int(time.time() * 1000)}"

        body = f"""
    id: {flow_id}
    namespace: {namespace}
    tasks:
      - id: hello
        type: io.kestra.plugin.core.log.Log
        message: Hello World!
    """
        self.kestra_client.flows.create_flow(tenant=self.tenant, body=body)

        resp = self.kestra_client.executions.create_execution(namespace=namespace, id=flow_id, wait=True, tenant=self.tenant)
        exec_id = getattr(resp, "id", None)
        assert exec_id is not None

        task_run_id = None
        if hasattr(resp, "task_run_list") and getattr(resp, "task_run_list"):
            task_run_id = getattr(getattr(resp, "task_run_list")[0], "id", None)

        assert task_run_id is not None

        state_request = ExecutionControllerStateRequest(
            taskRunId=task_run_id,
            state=StateType.FAILED
        )

        updated = self.kestra_client.executions.update_task_run_state(
            execution_id=exec_id,
            tenant=self.tenant,
            execution_controller_state_request=state_request
        )
        first_task_run = None
        if hasattr(updated, "task_run_list") and getattr(updated, "task_run_list"):
            first_task_run = getattr(updated, "task_run_list")[0]

        assert getattr(getattr(first_task_run, "state", None), "current", None) == "FAILED"


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Kestra EE

    All API operations, except for Superadmin-only endpoints, require a tenant identifier in the HTTP path.<br/> Endpoints designated as Superadmin-only are not tenant-scoped.

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest, pytest
from kestrapy import Configuration, KestraClient, KVControllerApiDeleteBulkRequest


class TestKVApi(unittest.TestCase):
    """KVApi unit test stubs"""

    def setUp(self) -> None:
        configuration = Configuration()
        configuration.host = "http://localhost:9902"
        configuration.username = "root@root.com"
        configuration.password = "Root!1234"

        self.kestra_client = KestraClient(configuration)
        self.tenant = "main"
        self.namespace = "test.namespace"

    def tearDown(self) -> None:
        pass

    def test_set_key_value(self) -> None:
        """Test case for set_key_value

        Puts a key-value pair in store
        """
        key = "test_set_key_value"
        value = "hello-kestra"

        self.kestra_client.kv.set_key_value(namespace=self.namespace, key=key, tenant=self.tenant, body=value)

        fetched = self.kestra_client.kv.key_value(namespace=self.namespace, key=key, tenant=self.tenant)
        assert getattr(fetched, 'value', None) is not None or fetched is not None

        try:
            self.kestra_client.kv.delete_key_value(namespace=self.namespace, key=key, tenant=self.tenant)
        except Exception:
            pass

    def test_get_key_value(self) -> None:
        """Test case for get_key_value

        Get value for a key
        """
        key = "test_get_key_value"
        value = "value-get"

        self.kestra_client.kv.set_key_value(namespace=self.namespace, key=key, tenant=self.tenant, body=value)
        fetched = self.kestra_client.kv.key_value(namespace=self.namespace, key=key, tenant=self.tenant)
        assert fetched is not None

    def test_list_keys(self) -> None:
        """Test case for list_keys

        List all keys for a namespace
        """
        key = "test_list_keys"
        value = "value-list"

        self.kestra_client.kv.set_key_value(namespace=self.namespace, key=key, tenant=self.tenant, body=value)

        entries = self.kestra_client.kv.list_keys(namespace=self.namespace, tenant=self.tenant)
        assert any(getattr(e, 'key', None) == key for e in entries)

    @pytest.mark.skip(reason="flaky with the run-test, works fine on 8080")
    def test_list_keys_with_inheritence(self) -> None:
        """Test case for list_keys_with_inheritence

        List all keys for inherited namespaces
        """
        key = "test_list_keys_with_inheritence"
        value = "value-inherited"
        parent_namespace = "test"
        child_namespace = f"{parent_namespace}.namespace"
        self.kestra_client.kv.set_key_value(namespace=parent_namespace, key=key, tenant=self.tenant, body=value)

        # Check with parent namespace before
        entries = self.kestra_client.kv.list_keys(namespace=parent_namespace, tenant=self.tenant)
        assert any(getattr(e, 'key', None) == key for e in entries)

        found = False
        entries = self.kestra_client.kv.list_keys_with_inheritence(namespace=child_namespace, tenant=self.tenant)
        if any(getattr(e, 'key', None) == key for e in entries):
            found = True

        assert found, f"key {key} not found after 3 attempts"

    def test_delete_key_value(self) -> None:
        """Test case for delete_key_value

        Delete a key-value pair
        """
        key = "test_delete_key_value"
        value = "to-delete"

        self.kestra_client.kv.set_key_value(namespace=self.namespace, key=key, tenant=self.tenant, body=value)

        deleted = self.kestra_client.kv.delete_key_value(namespace=self.namespace, key=key, tenant=self.tenant)
        assert deleted is True or deleted is None

        with self.assertRaises(Exception):
            self.kestra_client.kv.key_value(namespace=self.namespace, key=key, tenant=self.tenant)

    def test_delete_key_values(self) -> None:
        """Test case for delete_key_values

        Bulk-delete multiple key/value pairs from the given namespace.
        """
        key1 = "test_delete_key_values_1"
        key2 = "test_delete_key_values_2"
        self.kestra_client.kv.set_key_value(namespace=self.namespace, key=key1, tenant=self.tenant, body="v1")
        self.kestra_client.kv.set_key_value(namespace=self.namespace, key=key2, tenant=self.tenant, body="v2")

        req = KVControllerApiDeleteBulkRequest(keys=[key1, key2])
        resp = self.kestra_client.kv.delete_key_values(namespace=self.namespace, tenant=self.tenant, kv_controller_api_delete_bulk_request=req)

        assert resp is not None

        for k in (key1, key2):
            with self.assertRaises(Exception):
                self.kestra_client.kv.key_value(namespace=self.namespace, key=k, tenant=self.tenant)


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Kestra EE

    All API operations, except for Superadmin-only endpoints, require a tenant identifier in the HTTP path.<br/> Endpoints designated as Superadmin-only are not tenant-scoped.

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest
from kestrapy import Configuration, KestraClient, QueryFilter, QueryFilterOp, QueryFilterField
from datetime import datetime, timezone, timedelta
import time
from typing import Optional, Tuple
import uuid
from kestrapy import (
    Trigger,
    TriggerControllerSetDisabledRequest,
    Backfill,
)

import random


class TestTriggersApi(unittest.TestCase):
    """TriggersApi unit test stubs"""

    def setUp(self) -> None:
        configuration = Configuration()
        configuration.host = "http://localhost:9902"
        configuration.username = "root@root.com"
        configuration.password = "Root!1234"

        self.kestra_client = KestraClient(configuration)
        self.tenant = "main"

    def tearDown(self) -> None:
        pass

    def create_flow_with_trigger(self, flow_id: Optional[str] = None, trigger_id: Optional[str] = None, namespace: str = "test.triggers") -> Tuple[str, str]:
        """Create a flow (yaml source) with a single scheduled trigger.

        Returns (flow_id, trigger_id).
        """
        # generate unique ids if not provided
        if flow_id is None:
            flow_id = f"{uuid.uuid4().hex[:8]}"
        if trigger_id is None:
            trigger_id = f"trigger_{uuid.uuid4().hex[:8]}"

        # yaml source for the flow
        body = f"""id: {flow_id}
namespace: {namespace}

tasks:
  - id: hello
    type: io.kestra.plugin.core.flow.Sleep
    duration: PT5S

triggers:
  - id: {trigger_id}
    type: io.kestra.plugin.core.trigger.Schedule
    cron: "*/1 * * * *"
"""

        self.kestra_client.flows.create_flow(tenant=self.tenant, body=body)
        time.sleep(1)
        return flow_id, trigger_id

    def create_backfill_for_trigger(self, flow_id: str, trigger_id: str, start: Optional[datetime] = None, end: Optional[datetime] = None, namespace: str = "test.triggers") -> Trigger:
        """Attach a backfill to the given trigger by calling update_trigger.

        Returns the updated Trigger.
        """
        if start is None:
            start = (datetime.now(timezone.utc) - timedelta(hours=5)).isoformat()
        else:
            if isinstance(start, datetime):
                start = start.astimezone(timezone.utc).isoformat()

        if end is not None and isinstance(end, datetime):
            end = end.astimezone(timezone.utc).isoformat()

        backfill = Backfill(start=start, end=end)

        trigger = Trigger(namespace=namespace, flow_id=flow_id, trigger_id=trigger_id, var_date=datetime.now(timezone.utc).isoformat(), backfill=backfill, tenant_id=self.tenant)

        updated = self.kestra_client.triggers.update_trigger(tenant=self.tenant, trigger=trigger)
        return updated

    def test_delete_backfill(self) -> None:
        """Test case for delete_backfill

        Delete a backfill
        """
        flow_id, trigger_id = self.create_flow_with_trigger(flow_id=f"{self._testMethodName}", trigger_id=f"{self._testMethodName}_trigger")
        time.sleep(1) # ensure trigger is created
        self.create_backfill_for_trigger(flow_id=flow_id, trigger_id=trigger_id)
        trigger = Trigger(namespace="test.triggers", flow_id=flow_id, trigger_id=trigger_id, var_date=datetime.now(timezone.utc).isoformat(), tenant_id=self.tenant)

        resp = self.kestra_client.triggers.delete_backfill(tenant=self.tenant, trigger=trigger)
        assert resp is not None

    def test_delete_backfill_by_ids(self) -> None:
        """Test case for delete_backfill_by_ids

        Delete backfill for given triggers
        """
        flow_id, trigger_id = self.create_flow_with_trigger(flow_id=f"{self._testMethodName}", trigger_id=f"{self._testMethodName}_trigger")
        
        self.create_backfill_for_trigger(flow_id=flow_id, trigger_id=trigger_id)
        trigger = Trigger(namespace="test.triggers", flow_id=flow_id, trigger_id=trigger_id, var_date=datetime.now(timezone.utc).isoformat(), tenant_id=self.tenant)
        resp = self.kestra_client.triggers.delete_backfill_by_ids(tenant=self.tenant, trigger=[trigger])
        assert resp is not None

    def test_delete_backfill_by_query(self) -> None:
        """Test case for delete_backfill_by_query

        Delete backfill for given triggers
        """

        flow_id, trigger_id = self.create_flow_with_trigger(flow_id=f"{self._testMethodName}", trigger_id=f"{self._testMethodName}_trigger")
        self.create_backfill_for_trigger(flow_id=flow_id, trigger_id=trigger_id)
        qf = QueryFilter(
            field=QueryFilterField.TRIGGER_ID,
            operation=QueryFilterOp.CONTAINS,
            value={"value": self._testMethodName}
        )
        resp = self.kestra_client.triggers.delete_backfill_by_query(tenant=self.tenant, filters=[qf])
        assert resp is not None


    def test_disabled_triggers_by_ids(self) -> None:
        """Test case for disabled_triggers_by_ids

        Disable/enable given triggers
        """
        flow_id, trigger_id = self.create_flow_with_trigger(flow_id=f"{self._testMethodName}", trigger_id=f"{self._testMethodName}_trigger")
        trigger = Trigger(namespace="test.triggers", flow_id=flow_id, trigger_id=trigger_id, var_date=datetime.now(timezone.utc).isoformat(), tenant_id=self.tenant)
        req = TriggerControllerSetDisabledRequest(triggers=[trigger], disabled=True)
        resp = self.kestra_client.triggers.disabled_triggers_by_ids(tenant=self.tenant, trigger_controller_set_disabled_request=req)
        assert resp is not None

    def test_disabled_triggers_by_query(self) -> None:
        """Test case for disabled_triggers_by_query

        Disable/enable triggers by query parameters
        """
        self.create_flow_with_trigger(flow_id=f"{self._testMethodName}", trigger_id=f"{self._testMethodName}_trigger")
        qf = QueryFilter(
            field=QueryFilterField.TRIGGER_ID,
            operation=QueryFilterOp.CONTAINS,
            value={"value": self._testMethodName}
        )
        resp = self.kestra_client.triggers.disabled_triggers_by_query(disabled=True, tenant=self.tenant, filters=[qf])
        assert resp is not None

    def test_pause_backfill(self) -> None:
        """Test case for pause_backfill

        Pause a backfill
        """
        flow_id, trigger_id = self.create_flow_with_trigger(flow_id=f"{self._testMethodName}", trigger_id=f"{self._testMethodName}_trigger")
        
        self.create_backfill_for_trigger(flow_id=flow_id, trigger_id=trigger_id)
        trigger = Trigger(namespace="test.triggers", flow_id=flow_id, trigger_id=trigger_id, var_date=datetime.now(timezone.utc).isoformat(), tenant_id=self.tenant)
        resp = self.kestra_client.triggers.pause_backfill(tenant=self.tenant, trigger=trigger)
        assert resp is not None

    def test_pause_backfill_by_ids(self) -> None:
        """Test case for pause_backfill_by_ids

        Pause backfill for given triggers
        """
        flow_id, trigger_id = self.create_flow_with_trigger(flow_id=f"{self._testMethodName}", trigger_id=f"{self._testMethodName}_trigger")
        
        self.create_backfill_for_trigger(flow_id=flow_id, trigger_id=trigger_id)
        trigger = Trigger(namespace="test.triggers", flow_id=flow_id, trigger_id=trigger_id, var_date=datetime.now(timezone.utc).isoformat(), tenant_id=self.tenant)
        resp = self.kestra_client.triggers.pause_backfill_by_ids(tenant=self.tenant, trigger=[trigger])
        assert resp is not None

    def test_pause_backfill_by_query(self) -> None:
        """Test case for pause_backfill_by_query

        Pause backfill for given triggers
        """
        self.create_flow_with_trigger(flow_id=f"{self._testMethodName}", trigger_id=f"{self._testMethodName}_trigger")
        qf = QueryFilter(
            field=QueryFilterField.TRIGGER_ID,
            operation=QueryFilterOp.CONTAINS,
            value={"value": self._testMethodName}
        )
        resp = self.kestra_client.triggers.pause_backfill_by_query(tenant=self.tenant, filters=[qf])
        assert resp is not None

    def test_restart_trigger(self) -> None:
        """Test case for restart_trigger

        Restart a trigger
        """
        flow_id, trigger_id = self.create_flow_with_trigger(flow_id=f"{self._testMethodName}", trigger_id=f"{self._testMethodName}_trigger")
        # restart uses namespace/flowId/triggerId path params
        resp = self.kestra_client.triggers.restart_trigger(namespace="test.triggers", flow_id=flow_id, trigger_id=trigger_id, tenant=self.tenant)
        assert resp is not None

    def test_search_triggers(self) -> None:
        """Test case for search_triggers

        Search for triggers
        """
        self.create_flow_with_trigger(flow_id=f"{self._testMethodName}_{random.randint(1, 100)}", trigger_id=f"{self._testMethodName}_trigger_{random.randint(1, 100)}")
        time.sleep(1)
        qf = QueryFilter(
            field=QueryFilterField.TRIGGER_ID,
            operation=QueryFilterOp.CONTAINS,
            value={"value": self._testMethodName}
        )
        resp = self.kestra_client.triggers.search_triggers(page=1, size=10, tenant=self.tenant, filters=[qf])
        assert resp is not None

    def test_search_triggers_for_flow(self) -> None:
        """Test case for search_triggers_for_flow

        Get all triggers for a flow
        """
        flow_id, trigger_id = self.create_flow_with_trigger(flow_id=f"{self._testMethodName}", trigger_id=f"{self._testMethodName}_trigger")
        resp = self.kestra_client.triggers.search_triggers_for_flow(page=1, size=10, namespace="test.triggers", flow_id=flow_id, tenant=self.tenant)
        assert resp is not None

    def test_unlock_trigger(self) -> None:
        """Test case for unlock_trigger
        Difficult to have a trigger locked, so just assumed having the exception that raise that the trigger is not lock
        is enough to says that the endpoint is called correctly

        Unlock a trigger
        """
        flow_id, trigger_id = self.create_flow_with_trigger(flow_id=f"{self._testMethodName}", trigger_id=f"{self._testMethodName}_trigger")
        with self.assertRaises(Exception) as context:
            self.kestra_client.triggers.unlock_trigger(namespace="test.triggers", flow_id=flow_id, trigger_id=trigger_id, tenant=self.tenant)

        self.assertTrue('409' in str(context.exception))

    def test_unlock_triggers_by_ids(self) -> None:
        """Test case for unlock_triggers_by_ids

        Unlock given triggers
        """
        flow_id, trigger_id = self.create_flow_with_trigger(flow_id=f"{self._testMethodName}", trigger_id=f"{self._testMethodName}_trigger")
        trigger = Trigger(namespace="test.triggers", flow_id=flow_id, trigger_id=trigger_id, var_date=datetime.now(timezone.utc).isoformat(), tenant_id=self.tenant)
        resp = self.kestra_client.triggers.unlock_triggers_by_ids(tenant=self.tenant, trigger=[trigger])
        assert resp is not None

    def test_unlock_triggers_by_query(self) -> None:
        """Test case for unlock_triggers_by_query

        Unlock triggers by query parameters
        """
        self.create_flow_with_trigger(flow_id=f"{self._testMethodName}", trigger_id=f"{self._testMethodName}_trigger")

        qf = QueryFilter(
            field=QueryFilterField.TRIGGER_ID,
            operation=QueryFilterOp.CONTAINS,
            value={"value": self._testMethodName}
        )

        resp = self.kestra_client.triggers.unlock_triggers_by_query(tenant=self.tenant, filters=[qf])
        assert resp is not None

    def test_unpause_backfill(self) -> None:
        """Test case for unpause_backfill

        Unpause a backfill
        """
        flow_id, trigger_id = self.create_flow_with_trigger(flow_id=f"{self._testMethodName}", trigger_id=f"{self._testMethodName}_trigger")
        
        self.create_backfill_for_trigger(flow_id=flow_id, trigger_id=trigger_id)
        trigger = Trigger(namespace="test.triggers", flow_id=flow_id, trigger_id=trigger_id, var_date=datetime.now(timezone.utc).isoformat(), tenant_id=self.tenant)
        resp = self.kestra_client.triggers.unpause_backfill(tenant=self.tenant, trigger=trigger)
        assert resp is not None

    def test_unpause_backfill_by_ids(self) -> None:
        """Test case for unpause_backfill_by_ids

        Unpause backfill for given triggers
        """
        flow_id, trigger_id = self.create_flow_with_trigger(flow_id=f"{self._testMethodName}", trigger_id=f"{self._testMethodName}_trigger")
        
        self.create_backfill_for_trigger(flow_id=flow_id, trigger_id=trigger_id)
        trigger = Trigger(namespace="test.triggers", flow_id=flow_id, trigger_id=trigger_id, var_date=datetime.now(timezone.utc).isoformat(), tenant_id=self.tenant)
        resp = self.kestra_client.triggers.unpause_backfill_by_ids(tenant=self.tenant, trigger=[trigger])
        assert resp is not None

    def test_unpause_backfill_by_query(self) -> None:
        """Test case for unpause_backfill_by_query

        Unpause backfill for given triggers
        """
        self.create_flow_with_trigger(flow_id=f"{self._testMethodName}", trigger_id=f"{self._testMethodName}_trigger")

        qf = QueryFilter(
            field=QueryFilterField.TRIGGER_ID,
            operation=QueryFilterOp.CONTAINS,
            value={"value": self._testMethodName}
        )

        resp = self.kestra_client.triggers.unpause_backfill_by_query(tenant=self.tenant, filters=[qf])
        assert resp is not None

    def test_update_trigger(self) -> None:
        """Test case for update_trigger

        Update a trigger
        """
        flow_id, trigger_id = self.create_flow_with_trigger(flow_id=f"{self._testMethodName}", trigger_id=f"{self._testMethodName}_trigger")
        trigger = Trigger(namespace="test.triggers", flow_id=flow_id, trigger_id=trigger_id, var_date=datetime.now(timezone.utc).isoformat(), tenant_id=self.tenant)
        resp = self.kestra_client.triggers.update_trigger(tenant=self.tenant, trigger=trigger)
        assert resp is not None


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Kestra EE

    All API operations, except for Superadmin-only endpoints, require a tenant identifier in the HTTP path.<br/> Endpoints designated as Superadmin-only are not tenant-scoped.

    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import regex as re
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from kestrapy.models.schema_attribute_mutability import SchemaAttributeMutability
from kestrapy.models.schema_attribute_returned import SchemaAttributeReturned
from kestrapy.models.schema_attribute_type import SchemaAttributeType
from kestrapy.models.schema_attribute_uniqueness import SchemaAttributeUniqueness
from typing import Optional, Set
from typing_extensions import Self

class SchemaAttribute(BaseModel):
    """
    SchemaAttribute
    """ # noqa: E501
    attributes: Optional[List[SchemaAttribute]] = None
    urn: Optional[StrictStr] = None
    name: Optional[Annotated[str, Field(strict=True)]] = None
    type: Optional[SchemaAttributeType] = None
    sub_attributes: Optional[List[SchemaAttribute]] = Field(default=None, alias="subAttributes")
    multi_valued: Optional[StrictBool] = Field(default=None, alias="multiValued")
    description: Optional[StrictStr] = None
    required: Optional[StrictBool] = None
    canonical_values: Optional[List[StrictStr]] = Field(default=None, alias="canonicalValues")
    case_exact: Optional[StrictBool] = Field(default=None, alias="caseExact")
    mutability: Optional[SchemaAttributeMutability] = None
    returned: Optional[SchemaAttributeReturned] = None
    uniqueness: Optional[SchemaAttributeUniqueness] = None
    reference_types: Optional[List[StrictStr]] = Field(default=None, alias="referenceTypes")
    accessor: Optional[Dict[str, Any]] = None
    scim_resource_id_reference: Optional[StrictBool] = Field(default=None, alias="scimResourceIdReference")
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = ["attributes", "urn", "name", "type", "subAttributes", "multiValued", "description", "required", "canonicalValues", "caseExact", "mutability", "returned", "uniqueness", "referenceTypes", "accessor", "scimResourceIdReference"]

    @field_validator('name')
    def name_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"\p{Alpha}(-|_|\p{Alnum})*", value):
            raise ValueError(r"must validate the regular expression /\p{Alpha}(-|_|\p{Alnum})*/")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of SchemaAttribute from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set([
            "additional_properties",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in attributes (list)
        _items = []
        if self.attributes:
            for _item_attributes in self.attributes:
                if _item_attributes:
                    _items.append(_item_attributes.to_dict())
            _dict['attributes'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in sub_attributes (list)
        _items = []
        if self.sub_attributes:
            for _item_sub_attributes in self.sub_attributes:
                if _item_sub_attributes:
                    _items.append(_item_sub_attributes.to_dict())
            _dict['subAttributes'] = _items
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of SchemaAttribute from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "attributes": [SchemaAttribute.from_dict(_item) for _item in obj["attributes"]] if obj.get("attributes") is not None else None,
            "urn": obj.get("urn"),
            "name": obj.get("name"),
            "type": obj.get("type"),
            "subAttributes": [SchemaAttribute.from_dict(_item) for _item in obj["subAttributes"]] if obj.get("subAttributes") is not None else None,
            "multiValued": obj.get("multiValued"),
            "description": obj.get("description"),
            "required": obj.get("required"),
            "canonicalValues": obj.get("canonicalValues"),
            "caseExact": obj.get("caseExact"),
            "mutability": obj.get("mutability"),
            "returned": obj.get("returned"),
            "uniqueness": obj.get("uniqueness"),
            "referenceTypes": obj.get("referenceTypes"),
            "accessor": obj.get("accessor"),
            "scimResourceIdReference": obj.get("scimResourceIdReference")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj

# TODO: Rewrite to not use raise_errors
SchemaAttribute.model_rebuild(raise_errors=False)




# coding: utf-8

"""
    Kestra EE

    All API operations, except for Superadmin-only endpoints, require a tenant identifier in the HTTP path.<br/> Endpoints designated as Superadmin-only are not tenant-scoped.

    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class Type(str, Enum):
    """
    Type
    """

    """
    allowed enum values
    """
    STRING = 'STRING'
    ENUM = 'ENUM'
    SELECT = 'SELECT'
    INT = 'INT'
    FLOAT = 'FLOAT'
    BOOLEAN = 'BOOLEAN'
    BOOL = 'BOOL'
    DATETIME = 'DATETIME'
    DATE = 'DATE'
    TIME = 'TIME'
    DURATION = 'DURATION'
    FILE = 'FILE'
    JSON = 'JSON'
    URI = 'URI'
    SECRET = 'SECRET'
    ARRAY = 'ARRAY'
    MULTISELECT = 'MULTISELECT'
    YAML = 'YAML'
    EMAIL = 'EMAIL'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of Type from a JSON string"""
        return cls(json.loads(json_str))





# coding: utf-8

"""
    Kestra EE

    All API operations, except for Superadmin-only endpoints, require a tenant identifier in the HTTP path.<br/> Endpoints designated as Superadmin-only are not tenant-scoped.

    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import regex as re
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from kestrapy.models.assets_in_out import AssetsInOut
from kestrapy.models.state import State
from kestrapy.models.task_run_attempt import TaskRunAttempt
from typing import Optional, Set
from typing_extensions import Self

class TaskRun(BaseModel):
    """
    TaskRun
    """ # noqa: E501
    id: StrictStr
    execution_id: StrictStr = Field(alias="executionId")
    namespace: StrictStr
    flow_id: StrictStr = Field(alias="flowId")
    task_id: StrictStr = Field(alias="taskId")
    parent_task_run_id: Optional[StrictStr] = Field(default=None, alias="parentTaskRunId")
    value: Optional[StrictStr] = None
    attempts: Optional[List[TaskRunAttempt]] = None
    outputs: Optional[Dict[str, Any]] = None
    assets: Optional[AssetsInOut] = None
    state: State
    iteration: Optional[StrictInt] = None
    dynamic: Optional[StrictBool] = None
    force_execution: Optional[StrictBool] = Field(default=None, alias="forceExecution")
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = ["id", "executionId", "namespace", "flowId", "taskId", "parentTaskRunId", "value", "attempts", "outputs", "assets", "state", "iteration", "dynamic", "forceExecution"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of TaskRun from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set([
            "additional_properties",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in attempts (list)
        _items = []
        if self.attempts:
            for _item_attempts in self.attempts:
                if _item_attempts:
                    _items.append(_item_attempts.to_dict())
            _dict['attempts'] = _items
        # override the default output from pydantic by calling `to_dict()` of assets
        if self.assets:
            _dict['assets'] = self.assets.to_dict()
        # override the default output from pydantic by calling `to_dict()` of state
        if self.state:
            _dict['state'] = self.state.to_dict()
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        # set to None if outputs (nullable) is None
        # and model_fields_set contains the field
        if self.outputs is None and "outputs" in self.model_fields_set:
            _dict['outputs'] = None

        # set to None if assets (nullable) is None
        # and model_fields_set contains the field
        if self.assets is None and "assets" in self.model_fields_set:
            _dict['assets'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of TaskRun from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "executionId": obj.get("executionId"),
            "namespace": obj.get("namespace"),
            "flowId": obj.get("flowId"),
            "taskId": obj.get("taskId"),
            "parentTaskRunId": obj.get("parentTaskRunId"),
            "value": obj.get("value"),
            "attempts": [TaskRunAttempt.from_dict(_item) for _item in obj["attempts"]] if obj.get("attempts") is not None else None,
            "outputs": obj.get("outputs"),
            "assets": AssetsInOut.from_dict(obj["assets"]) if obj.get("assets") is not None else None,
            "state": State.from_dict(obj["state"]) if obj.get("state") is not None else None,
            "iteration": obj.get("iteration"),
            "dynamic": obj.get("dynamic"),
            "forceExecution": obj.get("forceExecution")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj


